@@ -66,6 +66,10 @@ tinymce.addI18n('tg',{
 "Restore last draft": "\u0422\u0430\u0493\u0438\u0440\u043e\u0442\u04b3\u043e\u0438 \u043e\u0445\u0438\u0440\u0438\u043d\u0440\u043e \u0431\u0435\u043a\u043e\u0440 \u043a\u0443\u043d",
 "Special character": "\u0420\u0430\u043c\u0437\u04b3\u043e\u0438 \u043c\u0430\u0445\u0441\u0443\u0441",
 "Source code": "\u041a\u043e\u0434\u0438 \u0430\u0441\u043b\u0438",
+"B": "B",
+"R": "R",
+"G": "G",
+"Color": "\u0420\u0430\u043d\u0433",
 "Right to left": "\u0410\u0437 \u0440\u043e\u0441\u0442 \u0431\u0430 \u0447\u0430\u043f",
 "Left to right": "\u0410\u0437 \u0447\u0430\u043f \u0431\u0430 \u0440\u043e\u0441\u0442",
 "Emoticons": "\u042d\u04b3\u0441\u043e\u0441\u04b3\u043e",
@@ -90,6 +94,25 @@ tinymce.addI18n('tg',{
 "Style": "\u0421\u0442\u0438\u043b",
 "Dimensions": "\u0427\u0435\u043d\u0430\u043a\u04b3\u043e",
 "Insert image": "\u0420\u0430\u0441\u043c \u0432\u043e\u0440\u0438\u0434 \u043a\u0430\u0440\u0434\u0430\u043d",
+"Zoom in": "Zoom in",
+"Contrast": "Contrast",
+"Back": "Back",
+"Gamma": "Gamma",
+"Flip horizontally": "Flip horizontally",
+"Resize": "Resize",
+"Sharpen": "Sharpen",
+"Zoom out": "Zoom out",
+"Image options": "Image options",
+"Apply": "Apply",
+"Brightness": "Brightness",
+"Rotate clockwise": "Rotate clockwise",
+"Rotate counterclockwise": "Rotate counterclockwise",
+"Edit image": "Edit image",
+"Color levels": "Color levels",
+"Crop": "Crop",
+"Orientation": "Orientation",
+"Flip vertically": "Flip vertically",
+"Invert": "Invert",
 "Insert date\/time": "\u0412\u043e\u0440\u0438\u0434\u0438 \u0420\u04ef\u0437\/\u0421\u043e\u0430\u0442",
 "Remove link": "\u041d\u0435\u0441\u0442 \u043a\u0430\u0440\u0434\u0430\u043d\u0438 \u0438\u0441\u0442\u0438\u043d\u043e\u0434",
 "Url": "Url",
@@ -100,7 +123,7 @@ tinymce.addI18n('tg',{
 "None": "\u04b2\u0435\u04b7 \u0447\u0438\u0437",
 "The URL you entered seems to be an external link. Do you want to add the required http:\/\/ prefix?": "URL \u0430\u0434\u0440\u0435\u0441\u0435, \u043a\u0438 \u0448\u0443\u043c\u043e \u0432\u043e\u0440\u0438\u0434 \u043a\u0430\u0440\u0434\u0435\u0434 \u0438\u0441\u0442\u0438\u043d\u043e\u0434 \u0430\u0441\u0442. \u041f\u0440\u0435\u0444\u0438\u043a\u0441\u0438 http:\/\/: \u0438\u043b\u043e\u0432\u0430 \u043a\u0443\u043d\u0430\u043c?",
 "Target": "\u041d\u0438\u0448\u043e\u043d\u0430",
-"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "URL \u0430\u0434\u0440\u0435\u0441\u0435, \u043a\u0438 \u0448\u0443\u043c\u043e \u0432\u043e\u0440\u0438\u0434 \u043a\u0430\u0440\u0434\u0435\u0434 Email \u0430\u0434\u0440\u0435\u0441 \u0430\u0441\u0442. \u043f\u0440\u0435\u0444\u0438\u043a\u0441\u0438 mailto: \u0438\u043b\u043e\u0432\u0430 \u043a\u0443\u043d\u0430\u043c",
+"The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "URL \u0430\u0434\u0440\u0435\u0441\u0435, \u043a\u0438 \u0448\u0443\u043c\u043e \u0432\u043e\u0440\u0438\u0434 \u043a\u0430\u0440\u0434\u0435\u0434 Email \u0430\u0434\u0440\u0435\u0441 \u0430\u0441\u0442. G\u0440\u0435\u0444\u0438\u043a\u0441\u0438 mailto: \u0438\u043b\u043e\u0432\u0430 \u043a\u0443\u043d\u0430\u043c?",
 "Insert\/edit link": "\u0412\u043e\u0440\u0438\u0434\/\u0442\u0430\u0493\u0438\u0440 \u0434\u043e\u0434\u0430\u043d\u0438 \u0438\u0441\u0442\u0438\u043d\u043e\u0434",
 "Insert\/edit video": "\u0412\u043e\u0440\u0438\u0434\/\u0442\u0430\u0493\u0438\u0440\u0438 \u0432\u0438\u0434\u0435\u043e",
 "Poster": "\u042d\u044a\u043b\u043e\u043d\u043d\u043e\u043c\u0430",
@@ -128,11 +151,13 @@ tinymce.addI18n('tg',{
 "Finish": "\u0422\u0430\u043c\u043e\u043c",
 "Ignore all": "\u0410\u04b3\u0430\u043c\u0438\u044f\u0442 \u043d\u0430\u0434\u043e\u0434\u0430\u043d \u0431\u0430 \u04b3\u0430\u043c\u043c\u0430\u0448",
 "Ignore": "\u0410\u04b3\u0430\u043c\u0438\u044f\u0442 \u043d\u0430\u0434\u043e\u0434\u0430\u043d",
+"Add to Dictionary": "\u0414\u0430\u0440 \u043b\u0443\u0493\u0430\u0442 \u0438\u043b\u043e\u0432\u0430 \u043a\u0443\u043d",
 "Insert row before": "\u0425\u0430\u0442 \u0432\u043e\u0440\u0438\u0434 \u043a\u0430\u0440\u0434\u0430\u043d \u043f\u0435\u0448 \u0430\u0437",
 "Rows": "\u0425\u0430\u0442\u04b3\u043e",
 "Height": "\u0411\u0430\u043b\u0430\u043d\u0434\u0438",
 "Paste row after": "\u0425\u0430\u0442 \u0433\u0443\u0437\u043e\u0448\u0442\u0430\u043d \u043f\u0430\u0441 \u0430\u0437",
 "Alignment": "\u0420\u043e\u0441\u0442 \u043a\u0430\u0440\u0434\u0430\u043d",
+"Border color": "\u0420\u0430\u043d\u0433\u0438 \u04b3\u0443\u0434\u0443\u0434",
 "Column group": "\u0413\u0443\u0440\u0443\u04b3\u0438 \u0441\u0443\u0442\u0443\u043d",
 "Row": "\u0425\u0430\u0442",
 "Insert column before": "\u0421\u0443\u0442\u0443\u043d \u0432\u043e\u0440\u0438\u0434 \u043a\u0430\u0440\u0434\u0430\u043d \u043f\u0435\u0448 \u0430\u0437",
@@ -176,6 +201,9 @@ tinymce.addI18n('tg',{
 "Insert template": "\u0412\u043e\u0440\u0438\u0434\u0438 \u049b\u043e\u043b\u0430\u0431",
 "Templates": "\u049a\u043e\u043b\u0430\u0431",
 "Background color": "\u0420\u0430\u043d\u0433\u0438 \u043f\u0443\u0448\u0442\u0438 \u043c\u0430\u0442\u043d",
+"Custom...": "\u0425\u0443\u0441\u0443\u0441\u04e3",
+"Custom color": "\u0420\u0430\u043d\u0433\u0438 \u0445\u0443\u0441\u0443\u0441\u04e3",
+"No color": "\u0411\u0435 \u0440\u0430\u043d\u0433",
 "Text color": "\u0420\u0430\u043d\u0433\u0438 \u043c\u0430\u0442\u043d",
 "Show blocks": "\u041d\u0438\u0448\u043e\u043d \u0434\u043e\u0434\u0430\u043d\u0438 \u0433\u0443\u0440\u0443\u04b3\u04b3\u043e",
 "Show invisible characters": "\u0420\u0430\u043c\u0437\u04b3\u043e\u0438 \u043c\u0430\u0445\u0441\u0443\u0441\u0440\u043e \u043d\u0438\u0448\u043e\u043d \u0434\u043e\u0434\u0430\u043d",
